@@ -37,10 +37,17 @@ int ccs_file_fcntl(struct file *file, unsigned int cmd, unsigned long arg)
 	return ccs_fcntl_permission(file, cmd, arg);
 }
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 19, 0)
+int ccs_file_open(struct file *file)
+{
+	return ccs_open_permission(file);
+}
+#else
 int ccs_file_open(struct file *file, const struct cred *cred)
 {
 	return ccs_open_permission(file);
 }
+#endif
 
 int ccs_socket_create(int family, int type, int protocol, int kern)
 {
