@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 0.2.2   2017/09/17
+ * Version: 0.2.3   2018/04/01
  */
 
 #include "internal.h"
@@ -104,7 +104,7 @@ struct cs_security *cs_find_task_security(const struct task_struct *task)
 		[hash_ptr((void *) task, CS_TASK_SECURITY_HASH_BITS)];
 	/* Make sure INIT_LIST_HEAD() in cs_mm_init() takes effect. */
 	while (!list->next)
-		;
+		smp_rmb();
 	rcu_read_lock();
 	list_for_each_entry_rcu(ptr, list, list) {
 		if (ptr->task != task)
