@@ -10,31 +10,8 @@
 #include "probe.h"
 
 /* Prototype definition. */
-static void cs_task_security_gc(void);
-static int cs_copy_cred_security(const struct cred *new,
-				  const struct cred *old, gfp_t gfp);
-static struct cs_security *cs_find_cred_security(const struct cred *cred);
-static DEFINE_SPINLOCK(cs_task_security_list_lock);
-static atomic_t cs_in_execve_tasks = ATOMIC_INIT(0);
-/*
- * List of "struct cs_security" for "struct pid".
- *
- * All instances on this list is guaranteed that "struct cs_security"->pid !=
- * NULL. Also, instances on this list that are in execve() are guaranteed that
- * "struct cs_security"->cred remembers "struct linux_binprm"->cred with a
- * refcount on "struct linux_binprm"->cred.
- */
-struct list_head cs_task_security_list[CS_MAX_TASK_SECURITY_HASH];
-/*
- * List of "struct cs_security" for "struct cred".
- *
- * Since the number of "struct cred" is nearly equals to the number of
- * "struct pid", we allocate hash tables like cs_task_security_list.
- *
- * All instances on this list are guaranteed that "struct cs_security"->pid ==
- * NULL and "struct cs_security"->cred != NULL.
- */
-static struct list_head cs_cred_security_list[CS_MAX_TASK_SECURITY_HASH];
+static int __cs_alloc_task_security(const struct task_struct *task);
+static void __cs_free_task_security(const struct task_struct *task);
 
 /* Dummy security context for avoiding NULL pointer dereference. */
 static struct cs_security cs_oom_security = {
@@ -46,13 +23,18 @@ static struct cs_security cs_default_security = {
 	.cs_domain_info = &cs_kernel_domain
 };
 
+/* List of "struct cs_security". */
+struct list_head cs_task_security_list[CS_MAX_TASK_SECURITY_HASH];
+/* Lock for protecting cs_task_security_list[]. */
+static DEFINE_SPINLOCK(cs_task_security_list_lock);
+
 /* For exporting variables and functions. */
 struct caitsith_exports caitsith_exports;
 
 /* Original hooks. */
 static union security_list_options original_cred_prepare;
-static union security_list_options original_cred_free;
-static union security_list_options original_cred_alloc_blank;
+static union security_list_options original_task_alloc;
+static union security_list_options original_task_free;
 
 #if !defined(CONFIG_CAITSITH_DEBUG)
 #define cs_debug_trace(pos) do { } while (0)
@@ -62,7 +44,7 @@ static union security_list_options original_cred_alloc_blank;
 		static bool done;					\
 		if (!done) {						\
 			printk(KERN_INFO				\
-			       "CAITSITH: Debug trace: " pos " of 4\n"); \
+			       "CAITSITH: Debug trace: " pos " of 2\n"); \
 			done = true;					\
 		}							\
 	} while (0)
@@ -78,116 +60,84 @@ static union security_list_options original_cred_alloc_blank;
  */
 static void cs_clear_execve(int ret, struct cs_security *security)
 {
-	struct cs_request_info *r;
+	struct cs_request_info *r = security->r;
 
-	if (security == &cs_default_security || security == &cs_oom_security)
+	if (security == &cs_default_security || security == &cs_oom_security ||
+	    !r)
 		return;
-	r = security->r;
 	security->r = NULL;
-	if (!r)
-		return;
-	atomic_dec(&cs_in_execve_tasks);
 	cs_finish_execve(ret, r);
 }
 
 /**
- * cs_rcu_free - RCU callback for releasing "struct cs_security".
+ * cs_task_alloc_security - Allocate memory for new tasks.
  *
- * @rcu: Pointer to "struct rcu_head".
+ * @p: Pointer to "struct task_struct".
+ * @clone_flags: Flags passed to clone().
  *
- * Returns nothing.
+ * Returns 0 on success, negative value otherwise.
  */
-static void cs_rcu_free(struct rcu_head *rcu)
+static int cs_task_alloc_security(struct task_struct *p,
+				   unsigned long clone_flags)
 {
-	struct cs_security *ptr = container_of(rcu, typeof(*ptr), rcu);
-	struct cs_request_info *r = ptr->r;
-	/*
-	 * If this security context was associated with "struct pid" and
-	 * ptr->cs_flags has CS_TASK_IS_IN_EXECVE set, it indicates that a
-	 * "struct task_struct" associated with this security context exited
-	 * immediately after do_execve() has failed.
-	 */
-	if (ptr->pid && (ptr->cs_flags & CS_TASK_IS_IN_EXECVE)) {
-		cs_debug_trace("1");
-		atomic_dec(&cs_in_execve_tasks);
+	int rc = __cs_alloc_task_security(p);
+
+	if (rc)
+		return rc;
+	if (original_task_alloc.task_alloc) {
+		rc = original_task_alloc.task_alloc(p, clone_flags);
+		if (rc)
+			__cs_free_task_security(p);
 	}
-	/*
-	 * If this security context was associated with "struct pid",
-	 * drop refcount obtained by get_pid() in cs_find_task_security().
-	 */
-	if (ptr->pid) {
-		cs_debug_trace("2");
-		put_pid(ptr->pid);
-	}
-	if (r) {
-		cs_debug_trace("3");
-		kfree(r->handler_path);
-		kfree(r);
-	}
-	kfree(ptr);
+	return rc;
 }
 
 /**
- * cs_del_security - Release "struct cs_security".
+ * cs_task_free_security - Release memory for "struct task_struct".
  *
- * @ptr: Pointer to "struct cs_security".
+ * @p: Pointer to "struct task_struct".
  *
  * Returns nothing.
  */
-static void cs_del_security(struct cs_security *ptr)
+static void cs_task_free_security(struct task_struct *p)
+{
+	struct cs_security *ptr = cs_find_task_security(p);
+	struct cs_request_info *r = ptr->r;
+
+	if (original_task_free.task_free)
+		original_task_free.task_free(p);
+	/*
+	 * Since an LSM hook for reverting domain transition is missing,
+	 * cs_finish_execve() is not called if exited immediately after
+	 * execve() failed.
+	 */
+	if (r) {
+		cs_debug_trace("2");
+		kfree(r->handler_path);
+		kfree(r);
+		ptr->r = NULL;
+	}
+	__cs_free_task_security(p);
+}
+
+/**
+ * __cs_free_task_security - Release memory associated with "struct task_struct".
+ *
+ * @task: Pointer to "struct task_struct".
+ *
+ * Returns nothing.
+ */
+static void __cs_free_task_security(const struct task_struct *task)
 {
 	unsigned long flags;
+	struct cs_security *ptr = cs_find_task_security(task);
 
 	if (ptr == &cs_default_security || ptr == &cs_oom_security)
 		return;
 	spin_lock_irqsave(&cs_task_security_list_lock, flags);
 	list_del_rcu(&ptr->list);
 	spin_unlock_irqrestore(&cs_task_security_list_lock, flags);
-	call_rcu(&ptr->rcu, cs_rcu_free);
-}
-
-/**
- * cs_add_cred_security - Add "struct cs_security" to list.
- *
- * @ptr: Pointer to "struct cs_security".
- *
- * Returns nothing.
- */
-static void cs_add_cred_security(struct cs_security *ptr)
-{
-	unsigned long flags;
-	struct list_head *list = &cs_cred_security_list
-		[hash_ptr((void *) ptr->cred, CS_TASK_SECURITY_HASH_BITS)];
-#ifdef CONFIG_CAITSITH_DEBUG
-	if (ptr->pid)
-		printk(KERN_INFO "CAITSITH: \"struct cs_security\"->pid != NULL\n");
-#endif
-	ptr->pid = NULL;
-	spin_lock_irqsave(&cs_task_security_list_lock, flags);
-	list_add_rcu(&ptr->list, list);
-	spin_unlock_irqrestore(&cs_task_security_list_lock, flags);
-}
-
-/**
- * cs_task_create - Make snapshot of security context for new task.
- *
- * @clone_flags: Flags passed to clone().
- *
- * Returns 0 on success, negative value otherwise.
- */
-static int cs_task_create(unsigned long clone_flags)
-{
-	struct cs_security *old_security;
-	struct cs_security *new_security;
-	struct cred *cred = prepare_creds();
-
-	if (!cred)
-		return -ENOMEM;
-	old_security = cs_find_task_security(current);
-	new_security = cs_find_cred_security(cred);
-	new_security->cs_domain_info = old_security->cs_domain_info;
-	new_security->cs_flags = old_security->cs_flags;
-	return commit_creds(cred);
+	kfree_rcu(ptr, rcu);
 }
 
 /**
@@ -202,7 +152,6 @@ static int cs_task_create(unsigned long clone_flags)
 static int cs_cred_prepare(struct cred *new, const struct cred *old,
 			   gfp_t gfp)
 {
-	int rc1;
 	/*
 	 * For checking whether reverting domain transition is needed or not.
 	 *
@@ -210,98 +159,11 @@ static int cs_cred_prepare(struct cred *new, const struct cred *old,
 	 */
 	if (gfp == GFP_KERNEL)
 		cs_find_task_security(current);
-	rc1 = cs_copy_cred_security(new, old, gfp);
-	if (gfp == GFP_KERNEL)
-		cs_task_security_gc();
-	if (original_cred_prepare.cred_prepare) {
-		const int rc2 = original_cred_prepare.cred_prepare(new, old,
-								   gfp);
-		if (rc2) {
-			cs_del_security(cs_find_cred_security(new));
-			return rc2;
-		}
-	}
-	return rc1;
-}
-
-/**
- * cs_cred_free - Release memory used by credentials.
- *
- * @cred: Pointer to "struct cred".
- *
- * Returns nothing.
- */
-static void cs_cred_free(struct cred *cred)
-{
-	if (original_cred_free.cred_free)
-		original_cred_free.cred_free(cred);
-	cs_del_security(cs_find_cred_security(cred));
-}
-
-/**
- * cs_alloc_cred_security - Allocate memory for new credentials.
- *
- * @cred: Pointer to "struct cred".
- * @gfp:  Memory allocation flags.
- *
- * Returns 0 on success, negative value otherwise.
- */
-static int cs_alloc_cred_security(const struct cred *cred, gfp_t gfp)
-{
-	struct cs_security *new_security = kzalloc(sizeof(*new_security),
-						    gfp);
-	if (!new_security)
-		return -ENOMEM;
-	new_security->cred = cred;
-	cs_add_cred_security(new_security);
+	if (original_cred_prepare.cred_prepare)
+		return original_cred_prepare.cred_prepare(new, old, gfp);
 	return 0;
 }
 
-/**
- * cs_cred_alloc_blank - Allocate memory for new credentials.
- *
- * @new: Pointer to "struct cred".
- * @gfp: Memory allocation flags.
- *
- * Returns 0 on success, negative value otherwise.
- */
-static int cs_cred_alloc_blank(struct cred *new, gfp_t gfp)
-{
-	const int rc1 = cs_alloc_cred_security(new, gfp);
-
-	if (original_cred_alloc_blank.cred_alloc_blank) {
-		const int rc2 = original_cred_alloc_blank.
-			cred_alloc_blank(new, gfp);
-		if (rc2) {
-			cs_del_security(cs_find_cred_security(new));
-			return rc2;
-		}
-	}
-	return rc1;
-}
-
-/**
- * cs_cred_transfer - Transfer "struct cs_security" between credentials.
- *
- * @new: Pointer to "struct cred".
- * @old: Pointer to "struct cred".
- *
- * Returns nothing.
- */
-static void cs_cred_transfer(struct cred *new, const struct cred *old)
-{
-	struct cs_security *new_security = cs_find_cred_security(new);
-	struct cs_security *old_security = cs_find_cred_security(old);
-
-	if (new_security == &cs_default_security ||
-	    new_security == &cs_oom_security ||
-	    old_security == &cs_default_security ||
-	    old_security == &cs_oom_security)
-		return;
-	new_security->cs_flags = old_security->cs_flags;
-	new_security->cs_domain_info = old_security->cs_domain_info;
-}
-
 /**
  * cs_bprm_committing_creds - A hook which is called when do_execve() succeeded.
  *
@@ -311,17 +173,7 @@ static void cs_cred_transfer(struct cred *new, const struct cred *old)
  */
 static void cs_bprm_committing_creds(struct linux_binprm *bprm)
 {
-	struct cs_security *old_security = cs_current_security();
-	struct cs_security *new_security;
-
-	if (old_security == &cs_default_security ||
-	    old_security == &cs_oom_security)
-		return;
-	cs_clear_execve(0, old_security);
-	/* Update current task's cred's domain for future fork(). */
-	new_security = cs_find_cred_security(bprm->cred);
-	new_security->cs_flags = old_security->cs_flags;
-	new_security->cs_domain_info = old_security->cs_domain_info;
+	cs_clear_execve(0, cs_current_security());
 }
 
 #ifndef CONFIG_CAITSITH_OMIT_USERSPACE_LOADER
@@ -397,7 +249,6 @@ static void cs_load_policy(const char *filename)
 static int cs_bprm_check_security(struct linux_binprm *bprm)
 {
 	struct cs_security *security = cs_current_security();
-	int rc;
 
 	if (security == &cs_default_security || security == &cs_oom_security)
 		return -ENOMEM;
@@ -407,12 +258,22 @@ static int cs_bprm_check_security(struct linux_binprm *bprm)
 	if (!cs_policy_loaded)
 		cs_load_policy(bprm->filename);
 #endif
-	rc = cs_start_execve(bprm, &security->r);
-	if (security->r)
-		atomic_inc(&cs_in_execve_tasks);
-	return rc;
+	return cs_start_execve(bprm, &security->r);
 }
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 19, 0)
+/**
+ * cs_file_open - Check permission for open().
+ *
+ * @f:    Pointer to "struct file".
+ *
+ * Returns 0 on success, negative value otherwise.
+ */
+static int cs_file_open(struct file *f)
+{
+	return cs_open_permission(&f->f_path, f->f_flags);
+}
+#else
 /**
  * cs_file_open - Check permission for open().
  *
@@ -425,6 +286,7 @@ static int cs_file_open(struct file *f, const struct cred *cred)
 {
 	return cs_open_permission(&f->f_path, f->f_flags);
 }
+#endif
 
 #ifdef CONFIG_SECURITY_PATH
 
@@ -1185,11 +1047,9 @@ static int cs_file_ioctl(struct file *filp, unsigned int cmd,
 
 static struct security_hook_list caitsith_hooks[] = {
 	/* Security context allocator. */
-	MY_HOOK_INIT(cred_free, cs_cred_free),
+	MY_HOOK_INIT(task_free, cs_task_free_security),
 	MY_HOOK_INIT(cred_prepare, cs_cred_prepare),
-	MY_HOOK_INIT(cred_alloc_blank, cs_cred_alloc_blank),
-	MY_HOOK_INIT(cred_transfer, cs_cred_transfer),
-	MY_HOOK_INIT(task_create, cs_task_create),
+	MY_HOOK_INIT(task_alloc, cs_task_alloc_security),
 	/* Security context updater for successful execve(). */
 	MY_HOOK_INIT(bprm_check_security, cs_bprm_check_security),
 	MY_HOOK_INIT(bprm_committing_creds, cs_bprm_committing_creds),
@@ -1242,12 +1102,32 @@ static struct security_hook_list caitsith_hooks[] = {
 
 static inline void add_hook(struct security_hook_list *hook)
 {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 17, 0)
+	hlist_add_tail_rcu(&hook->list, hook->head);
+#else
 	list_add_tail_rcu(&hook->list, hook->head);
+#endif
 }
 
 static void __init swap_hook(struct security_hook_list *hook,
 			     union security_list_options *original)
 {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 17, 0)
+	struct hlist_head *list = hook->head;
+
+	if (hlist_empty(list)) {
+		add_hook(hook);
+	} else {
+		struct security_hook_list *shp =
+			hlist_entry(list->first, typeof(*shp), list);
+
+		while (shp->list.next)
+			shp = hlist_entry(shp->list.next, typeof(*shp), list);
+		*original = shp->hook;
+		smp_wmb();
+		shp->hook = hook->hook;
+	}
+#else
 	struct list_head *list = hook->head;
 
 	if (list_empty(list)) {
@@ -1255,12 +1135,96 @@ static void __init swap_hook(struct security_hook_list *hook,
 	} else {
 		struct security_hook_list *shp =
 			list_last_entry(list, struct security_hook_list, list);
+
 		*original = shp->hook;
 		smp_wmb();
 		shp->hook = hook->hook;
 	}
+#endif
 }
 
+#if defined(CONFIG_STRICT_KERNEL_RWX) && !defined(CONFIG_SECURITY_WRITABLE_HOOKS)
+#include <linux/uaccess.h> /* probe_kernel_write() */
+#define NEED_TO_CHECK_HOOKS_ARE_WRITABLE
+
+#if defined(CONFIG_X86)
+#define MAX_RO_PAGES 1024
+static struct page *ro_pages[MAX_RO_PAGES] __initdata;
+static unsigned int ro_pages_len __initdata;
+
+static bool __init lsm_test_page_ro(void *addr)
+{
+	unsigned int i;
+	int unused;
+	struct page *page;
+
+	page = (struct page *) lookup_address((unsigned long) addr, &unused);
+	if (!page)
+		return false;
+	if (test_bit(_PAGE_BIT_RW, &(page->flags)))
+		return true;
+	for (i = 0; i < ro_pages_len; i++)
+		if (page == ro_pages[i])
+			return true;
+	if (ro_pages_len == MAX_RO_PAGES)
+		return false;
+	ro_pages[ro_pages_len++] = page;
+	return true;
+}
+
+static bool __init check_ro_pages(struct security_hook_heads *hooks)
+{
+	int i;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 17, 0)
+	struct hlist_head *list = &hooks->capable;
+
+	if (!probe_kernel_write(list, list, sizeof(void *)))
+		return true;
+	for (i = 0; i < ARRAY_SIZE(caitsith_hooks); i++) {
+		struct hlist_head *head = caitsith_hooks[i].head;
+		struct security_hook_list *shp;
+
+		if (!lsm_test_page_ro(&head->first))
+			return false;
+		hlist_for_each_entry(shp, head, list)
+			if (!lsm_test_page_ro(&shp->list.next) ||
+			    !lsm_test_page_ro(&shp->list.pprev))
+				return false;
+	}
+#else
+	struct list_head *list = &hooks->capable;
+
+	if (!probe_kernel_write(list, list, sizeof(void *)))
+		return true;
+	for (i = 0; i < ARRAY_SIZE(caitsith_hooks); i++) {
+		struct list_head *head = caitsith_hooks[i].head;
+		struct security_hook_list *shp;
+
+		if (!lsm_test_page_ro(&head->next) ||
+		    !lsm_test_page_ro(&head->prev))
+			return false;
+		list_for_each_entry(shp, head, list)
+			if (!lsm_test_page_ro(&shp->list.next) ||
+			    !lsm_test_page_ro(&shp->list.prev))
+				return false;
+	}
+#endif
+	return true;
+}
+#else
+static bool __init check_ro_pages(struct security_hook_heads *hooks)
+{
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 17, 0)
+	struct hlist_head *list = &hooks->capable;
+#else
+	struct list_head *list = &hooks->capable;
+#endif
+
+	return !probe_kernel_write(list, list, sizeof(void *));
+}
+#endif
+#endif
+
 /**
  * cs_init - Initialize this module.
  *
@@ -1277,6 +1241,12 @@ static int __init cs_init(void)
 		caitsith_hooks[idx].head = ((void *) hooks)
 			+ ((unsigned long) caitsith_hooks[idx].head)
 			- ((unsigned long) &probe_dummy_security_hook_heads);
+#if defined(NEED_TO_CHECK_HOOKS_ARE_WRITABLE)
+	if (!check_ro_pages(hooks)) {
+		printk(KERN_INFO "Can't update security_hook_heads due to write protected. Retry with rodata=0 kernel command line option added.\n");
+		return -EINVAL;
+	}
+#endif
 	caitsith_exports.find_task_by_vpid = probe_find_task_by_vpid();
 	if (!caitsith_exports.find_task_by_vpid)
 		goto out;
@@ -1286,16 +1256,22 @@ static int __init cs_init(void)
 	caitsith_exports.d_absolute_path = probe_d_absolute_path();
 	if (!caitsith_exports.d_absolute_path)
 		goto out;
-	for (idx = 0; idx < CS_MAX_TASK_SECURITY_HASH; idx++) {
-		INIT_LIST_HEAD(&cs_cred_security_list[idx]);
+	for (idx = 0; idx < CS_MAX_TASK_SECURITY_HASH; idx++)
 		INIT_LIST_HEAD(&cs_task_security_list[idx]);
-	}
 	cs_init_module();
-	swap_hook(&caitsith_hooks[0], &original_cred_free);
+#if defined(NEED_TO_CHECK_HOOKS_ARE_WRITABLE) && defined(CONFIG_X86)
+	for (idx = 0; idx < ro_pages_len; idx++)
+		set_bit(_PAGE_BIT_RW, &(ro_pages[idx]->flags));
+#endif
+	swap_hook(&caitsith_hooks[0], &original_task_free);
 	swap_hook(&caitsith_hooks[1], &original_cred_prepare);
-	swap_hook(&caitsith_hooks[2], &original_cred_alloc_blank);
+	swap_hook(&caitsith_hooks[2], &original_task_alloc);
 	for (idx = 3; idx < ARRAY_SIZE(caitsith_hooks); idx++)
 		add_hook(&caitsith_hooks[idx]);
+#if defined(NEED_TO_CHECK_HOOKS_ARE_WRITABLE) && defined(CONFIG_X86)
+	for (idx = 0; idx < ro_pages_len; idx++)
+		clear_bit(_PAGE_BIT_RW, &(ro_pages[idx]->flags));
+#endif
 	return 0;
 out:
 	return -EINVAL;
@@ -1315,21 +1291,6 @@ MODULE_LICENSE("GPL");
  */
 bool cs_used_by_cred(const struct cs_domain_info *domain)
 {
-	int idx;
-	struct cs_security *ptr;
-
-	for (idx = 0; idx < CS_MAX_TASK_SECURITY_HASH; idx++) {
-		struct list_head *list = &cs_cred_security_list[idx];
-
-		list_for_each_entry_rcu(ptr, list, list) {
-			struct cs_request_info *r = ptr->r;
-
-			if (ptr->cs_domain_info == domain ||
-			    (r && r->previous_domain == domain)) {
-				return true;
-			}
-		}
-	}
 	return false;
 }
 
@@ -1351,6 +1312,30 @@ static void cs_add_task_security(struct cs_security *ptr,
 	spin_unlock_irqrestore(&cs_task_security_list_lock, flags);
 }
 
+/**
+ * __cs_alloc_task_security - Allocate memory for new tasks.
+ *
+ * @task: Pointer to "struct task_struct".
+ *
+ * Returns 0 on success, negative value otherwise.
+ */
+static int __cs_alloc_task_security(const struct task_struct *task)
+{
+	struct cs_security *old_security = cs_current_security();
+	struct cs_security *new_security = kzalloc(sizeof(*new_security),
+						   GFP_KERNEL);
+	struct list_head *list = &cs_task_security_list
+		[hash_ptr((void *) task, CS_TASK_SECURITY_HASH_BITS)];
+
+	if (!new_security)
+		return -ENOMEM;
+	new_security->task = task;
+	new_security->cs_domain_info = old_security->cs_domain_info;
+	new_security->cs_flags = old_security->cs_flags;
+	cs_add_task_security(new_security, list);
+	return 0;
+}
+
 /**
  * cs_find_task_security - Find "struct cs_security" for given task.
  *
@@ -1374,7 +1359,7 @@ struct cs_security *cs_find_task_security(const struct task_struct *task)
 		smp_rmb();
 	rcu_read_lock();
 	list_for_each_entry_rcu(ptr, list, list) {
-		if (ptr->pid != task->pids[PIDTYPE_PID].pid)
+		if (ptr->task != task)
 			continue;
 		rcu_read_unlock();
 		/*
@@ -1402,33 +1387,14 @@ struct cs_security *cs_find_task_security(const struct task_struct *task)
 		if (task == current &&
 		    (ptr->cs_flags & CS_TASK_IS_IN_EXECVE) &&
 		    !current->in_execve) {
-			cs_debug_trace("4");
+			cs_debug_trace("1");
 			cs_clear_execve(-1, ptr);
 		}
 		return ptr;
 	}
 	rcu_read_unlock();
-	if (task != current) {
-		/*
-		 * If a thread does nothing after fork(), caller will reach
-		 * here because "struct cs_security" for that thread is not
-		 * yet allocated. But that thread is keeping a snapshot of
-		 * "struct cs_security" taken as of cs_task_create()
-		 * associated with that thread's "struct cred".
-		 *
-		 * Since that snapshot will be used as initial data when that
-		 * thread allocates "struct cs_security" for that thread, we
-		 * can return that snapshot rather than &cs_default_security.
-		 *
-		 * Since this function is called by only cs_select_one() and
-		 * cs_read_pid() (via cs_task_domain() and cs_task_flags()),
-		 * it is guaranteed that caller has called rcu_read_lock()
-		 * (via cs_tasklist_lock()) before finding this thread and
-		 * this thread is valid. Therefore, we can do __task_cred(task)
-		 * like get_robust_list() does.
-		 */
-		return cs_find_cred_security(__task_cred(task));
-	}
+	if (task != current)
+		return &cs_default_security;
 	/* Use GFP_ATOMIC because caller may have called rcu_read_lock(). */
 	ptr = kzalloc(sizeof(*ptr), GFP_ATOMIC);
 	if (!ptr) {
@@ -1437,102 +1403,8 @@ struct cs_security *cs_find_task_security(const struct task_struct *task)
 		send_sig(SIGKILL, current, 0);
 		return &cs_oom_security;
 	}
-	*ptr = *cs_find_cred_security(task->cred);
-	/* We can shortcut because task == current. */
-	ptr->pid = get_pid(((struct task_struct *) task)->
-			   pids[PIDTYPE_PID].pid);
-	ptr->cred = NULL;
+	*ptr = cs_default_security;
+	ptr->task = task;
 	cs_add_task_security(ptr, list);
 	return ptr;
 }
-
-/**
- * cs_copy_cred_security - Allocate memory for new credentials.
- *
- * @new: Pointer to "struct cred".
- * @old: Pointer to "struct cred".
- * @gfp: Memory allocation flags.
- *
- * Returns 0 on success, negative value otherwise.
- */
-static int cs_copy_cred_security(const struct cred *new,
-				  const struct cred *old, gfp_t gfp)
-{
-	struct cs_security *old_security = cs_find_cred_security(old);
-	struct cs_security *new_security =
-		kzalloc(sizeof(*new_security), gfp);
-	if (!new_security)
-		return -ENOMEM;
-	*new_security = *old_security;
-	new_security->cred = new;
-	cs_add_cred_security(new_security);
-	return 0;
-}
-
-/**
- * cs_find_cred_security - Find "struct cs_security" for given credential.
- *
- * @cred: Pointer to "struct cred".
- *
- * Returns pointer to "struct cs_security" on success, &cs_default_security
- * otherwise.
- */
-static struct cs_security *cs_find_cred_security(const struct cred *cred)
-{
-	struct cs_security *ptr;
-	struct list_head *list = &cs_cred_security_list
-		[hash_ptr((void *) cred, CS_TASK_SECURITY_HASH_BITS)];
-	rcu_read_lock();
-	list_for_each_entry_rcu(ptr, list, list) {
-		if (ptr->cred != cred)
-			continue;
-		rcu_read_unlock();
-		return ptr;
-	}
-	rcu_read_unlock();
-	return &cs_default_security;
-}
-
-/**
- * cs_task_security_gc - Do garbage collection for "struct task_struct".
- *
- * Returns nothing.
- *
- * Since security_task_free() is missing, I can't release memory associated
- * with "struct task_struct" when a task dies. Therefore, I hold a reference on
- * "struct pid" and runs garbage collection when associated
- * "struct task_struct" has gone.
- */
-static void cs_task_security_gc(void)
-{
-	static DEFINE_SPINLOCK(lock);
-	static atomic_t gc_counter = ATOMIC_INIT(0);
-	unsigned int idx;
-	/*
-	 * If some process is doing execve(), try to garbage collection now.
-	 * We should kfree() memory associated with "struct cs_security"->r
-	 * as soon as execve() has completed in order to compensate for lack of
-	 * security_bprm_free() and security_task_free() hooks.
-	 *
-	 * Otherwise, reduce frequency for performance reason.
-	 */
-	if (!atomic_read(&cs_in_execve_tasks) &&
-	    atomic_inc_return(&gc_counter) < 1024)
-		return;
-	if (!spin_trylock(&lock))
-		return;
-	atomic_set(&gc_counter, 0);
-	rcu_read_lock();
-	for (idx = 0; idx < CS_MAX_TASK_SECURITY_HASH; idx++) {
-		struct cs_security *ptr;
-		struct list_head *list = &cs_task_security_list[idx];
-
-		list_for_each_entry_rcu(ptr, list, list) {
-			if (pid_task(ptr->pid, PIDTYPE_PID))
-				continue;
-			cs_del_security(ptr);
-		}
-	}
-	rcu_read_unlock();
-	spin_unlock(&lock);
-}
